"""
MailBoxer for Japanese
"""
__version__ = "0.2.0"

import StringIO, re, difflib
import multifile, mimetools, mimify
from DateTime import DateTime
from Globals import InitializeClass, DTMLFile
import jplib
from Products.MailBoxer.MailBoxer import MailBoxer

def manage_addMail(self, Mail):
    """ store mail & attachments in a folder and return it """

    (mailHeader, mailBody) = self.splitMail(Mail)

    # always take our own date... don't believe the clients!
    if self.keepdate:
        timetuple = rfc822.parsedate_tz(mailHeader.get('date'))
        time = DateTime(rfc822.mktime_tz(timetuple))
    else:
        time = DateTime()
    
    # now let's create the date-path (yyyy/yyyy-mm) 
    year  = str(time.year())                  # yyyy
    month = "%s-%s" % (year, str(time.mm()))  # yyyy-mm

    archive = getattr(self,'archive')
    
    # do we have a year folder already?
    if not hasattr(archive, year):
        archive.manage_addFolder(year, title=year)
    yearFolder=getattr(archive, year)

    # do we have a month folder already?
    if not hasattr(yearFolder, month):
        yearFolder.manage_addFolder(month, title=month)
    monthFolder=getattr(yearFolder, month)

    # let's create the mailObject
    mailFolder = monthFolder
    
    Subject = self.mime_decode_header(mailHeader.get('subject',
                                                     'No Subject'))
    From = self.mime_decode_header(mailHeader.get('from','No From'))
    Title = "%s / %s" % (Subject, From)

    # maybe it's a reply ?
    if ':' in Subject:
        for currentFolder in monthFolder.objectValues(['Folder']):
            if difflib.get_close_matches(Subject,
                                         [currentFolder.mailSubject]):
                mailFolder=currentFolder

    # search a free id for the mailobject
    id = time.millis()
    while hasattr(mailFolder,str(id)):  
         id = id + 1
         
    id = str(id)

    mailFolder.manage_addFolder(id, title=Title)
    mailObject = getattr(mailFolder, id)

    # unpack attachments
    (TextBody, ContentType, HtmlBody) = self._unpackMultifile(mailObject, 
                                                 multifile.MultiFile(
                                                  StringIO.StringIO(Mail)))

    # ContentType is only set for the TextBody
    if ContentType:
        mailBody = TextBody
    else:
        mailBody = self.HtmlToText(HtmlBody)

    try:
        mailBody = self._convJP(mailBody)
    except:
        pass

    # and now add some properties to our new mailobject 
    mailObject.manage_addProperty('mailFrom', From, 'string')
    mailObject.manage_addProperty('mailSubject', Subject, 'string')
    mailObject.manage_addProperty('mailDate', time, 'date')
    mailObject.manage_addProperty('mailBody', mailBody, 'text')
    
    # insert header if a regular expression is set and matches
    if self.headers:
        msg = mimetools.Message(StringIO.StringIO(Mail))
        headers = []
        for (key,value) in msg.items():
            if re.match(self.headers, key, re.IGNORECASE):
                headers.append('%s: %s' % (key, value.strip()))
        
        mailObject.manage_addProperty('mailHeader', headers, 'lines')

    # Index the new created mailFolder in the catalog
    if hasattr(self, self.catalog):
        getattr(self, self.catalog).catalog_object(mailObject)

    return mailObject

def mime_decode_header(self, strg):
    # Returns the undecoded header
    try:
        return self._convJP(strg)
    except:
        return mimify.mime_decode_header(strg)

def _unpackMultifile(self, obj, multifile):
    """ Unpack body and attachments; return (TextBody, ContentType, HtmlBody). """

    TextBody = HtmlBody = ContentType = ''

    msg = mimetools.Message(multifile)
    maintype = msg.getmaintype()
    subtype = msg.getsubtype()

    name = msg.getparam('name')
    
    if not name:
        # Check for disposition header (RFC:1806)
        disposition = msg.getheader('Content-Disposition')
        if disposition:
            matchObj = re.search('(?i)filename="*(?P<filename>[^\s"]*)"*', 
                                   disposition)
            if matchObj:
                name = matchObj.group('filename')

    # Recurse over all nested multiparts
    if maintype == 'multipart':

        multifile.push(msg.getparam('boundary'))
        multifile.readlines()
        while not multifile.last:
            multifile.next()
                
            (tmpTextBody, tmpContentType, tmpHtmlBody) = \
                                       self._unpackMultifile(obj, multifile)
                
            # Return ContentType only for the plain-body of a mail
            if tmpContentType and not TextBody:
                TextBody = tmpTextBody
                ContentType = tmpContentType
                
            if tmpHtmlBody:
                HtmlBody = tmpHtmlBody
                    
        multifile.pop()
        return (TextBody,ContentType,HtmlBody)

    # Process MIME-encoded data
    plainfile = StringIO.StringIO()

    try:
        mimetools.decode(multifile,plainfile,msg.getencoding())
    # unknown or no encoding? 7bit, 8bit or whatever... copy literal
    except ValueError:
        mimetools.copyliteral(multifile,plainfile)

    body = plainfile.getvalue()
    plainfile.close()
    
    # Get plain text
    if maintype == 'text' and subtype == 'plain' and not name:
        TextBody = body
        ContentType = msg.get('content-type', 'text/plain')
            
    # Decode the rest
    else:
        # No name? This should be the html-body...
        if not name:
            name = '%s.%s' % (maintype,subtype)
            HtmlBody = body

        if obj and self.archived=='with attachments':
           
            id = DateTime().millis()
            # to be sure: test and search for a free id...
            while hasattr(obj,str("%s.%s" % (str(id), subtype))):  
                id = id + 1
                    
            id = "%s.%s" % (str(id), subtype)
            obj.manage_addFile(id, title = self.mime_decode_header(name),
                                   file = body,
                                   content_type = maintype + '/' + subtype)

    return (TextBody, ContentType, HtmlBody)

def _convJP(self, strg):
    # get management_page_charset
    try:
        charset = self.management_page_charset
    except KeyError, e:
        raise KeyError, e
    except Exception, e:
        raise Exception, e

    # mailBody charset convert to management_page_charset
    if re.match('euc-*jp|ujis', charset, re.I):
        return jplib.euc(strg)
    elif re.match('s.*jis', charset, re.I):
        return jplib.sjis(strg)
    elif re.match('utf-*8', charset, re.I):
        return jplib.utf8(strg)
    elif re.match('iso-2022-jp|jis', charset, re.I):
        return jplib.jis(strg)
    else:
        return strg

MailBoxer.manage_addMail = manage_addMail
MailBoxer.mime_decode_header = mime_decode_header
MailBoxer._unpackMultifile = _unpackMultifile
MailBoxer._convJP = _convJP
InitializeClass(MailBoxer)
